<?php

declare(strict_types=1);

namespace Drupal\entity_type_behaviors\Plugin\GraphQLCompose\SchemaType;

use Drupal\entity_type_behaviors\Config\BehaviorConfigFactory;
use Drupal\graphql_compose\Plugin\GraphQLCompose\GraphQLComposeSchemaTypeBase;
use GraphQL\Type\Definition\UnionType;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * EntityTypeBehaviorUnions GraphQL Compose Schema Type.
 *
 * @GraphQLComposeSchemaType(
 *   id = "EntityTypeBehaviorUnions",
 * )
 */
class EntityTypeBehaviorUnions extends GraphQLComposeSchemaTypeBase {

  /**
   * The behavior configuration factory.
   *
   * @var \Drupal\entity_type_behaviors\Config\BehaviorConfigFactory
   */
  private BehaviorConfigFactory $behaviorConfigFactory;

  /**
   * Map of enabled behaviors for entity types and bundles.
   *
   * @var array
   */
  private array $enabledBehaviorsMap = [];

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): self {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->behaviorConfigFactory = $container->get('entity_type_behaviors.config.factory');
    $instance->buildEnabledBehaviorsMap();
    return $instance;
  }

  /**
   * Build a map of enabled behaviors for each entity type/bundle.
   */
  private function buildEnabledBehaviorsMap(): void {
    $entityTypeBundles = $this->getEntityTypesWithBehaviors();

    foreach ($entityTypeBundles as $entityTypeId => $bundles) {
      foreach ($bundles as $bundleId => $enabledBehaviors) {
        $key = $this->getBehaviorMapKey($entityTypeId, $bundleId);
        $this->enabledBehaviorsMap[$key] = $enabledBehaviors;
      }
    }
  }

  /**
   * Get the key for the behavior map.
   *
   * @param string $entityTypeId
   *   The entity type ID.
   * @param string $bundleId
   *   The bundle ID.
   *
   * @return string
   *   The behavior map key.
   */
  private function getBehaviorMapKey(string $entityTypeId, string $bundleId): string {
    return "$entityTypeId:$bundleId";
  }

  /**
   * {@inheritdoc}
   */
  public function getTypes(): array {
    $types = [];
    $entityTypeBundles = $this->getEntityTypesWithBehaviors();

    foreach ($entityTypeBundles as $entityTypeId => $bundles) {
      foreach ($bundles as $bundleId => $enabledBehaviors) {
        if (empty($enabledBehaviors)) {
          continue;
        }

        $types[] = $this->createBehaviorUnionType($entityTypeId, $bundleId, $enabledBehaviors);
      }
    }

    return $types;
  }

  /**
   * Create a behavior union type.
   *
   * @param string $entityTypeId
   *   The entity type ID.
   * @param string $bundleId
   *   The bundle ID.
   * @param array $enabledBehaviors
   *   The array of enabled behavior IDs.
   *
   * @return \GraphQL\Type\Definition\UnionType
   *   The created union type.
   */
  private function createBehaviorUnionType(string $entityTypeId, string $bundleId, array $enabledBehaviors): UnionType {
    $unionTypeName = 'EntityTypeBehaviors' . $this->pascalCase($entityTypeId) . $this->pascalCase($bundleId);
    $unionTypes = array_map(
      fn($behaviorId) => 'EntityTypeBehavior' . $this->pascalCase($behaviorId),
      $enabledBehaviors
    );

    return new UnionType([
      'name' => $unionTypeName,
      'description' => (string) $this->t('Entity type behaviors for @entityType:@bundle', [
        '@entityType' => $entityTypeId,
        '@bundle' => $bundleId,
      ]),
      'types' => function () use ($unionTypes) {
        return array_map(fn($typeName) => static::type($typeName), $unionTypes);
      },
      'resolveType' => function ($value) use ($entityTypeId, $bundleId) {
        $behaviorId = $value['id'] ?? NULL;
        if (!$behaviorId) {
          return NULL;
        }

        $key = $this->getBehaviorMapKey($entityTypeId, $bundleId);
        if (!isset($this->enabledBehaviorsMap[$key]) || !in_array($behaviorId, $this->enabledBehaviorsMap[$key])) {
          return NULL;
        }

        // Set the behaviorType field.
        $value['behaviorType'] = $behaviorId;
        return static::type('EntityTypeBehavior' . $this->pascalCase($behaviorId));
      },
    ]);
  }

  /**
   * Get entity types with behaviors.
   *
   * @return array
   *   An associative array of entity types and their bundles
   *   with enabled behaviors.
   */
  private function getEntityTypesWithBehaviors(): array {
    $result = [];
    $entityTypeBundles = $this->behaviorConfigFactory->getConfiguredEntityTypesAndBundles();

    foreach ($entityTypeBundles as $entityTypeId => $bundles) {
      foreach ($bundles as $bundleId) {
        $enabledBehaviors = $this->behaviorConfigFactory->getEnabledConfiguredBehaviors($entityTypeId, $bundleId);
        if (!empty($enabledBehaviors)) {
          $result[$entityTypeId][$bundleId] = array_keys($enabledBehaviors);
        }
      }
    }

    return $result;
  }

  /**
   * Convert string to PascalCase.
   *
   * @param string $string
   *   The string to convert.
   *
   * @return string
   *   The PascalCase string.
   */
  private function pascalCase(string $string): string {
    return str_replace(' ', '', ucwords(str_replace('_', ' ', $string)));
  }

}
