<?php

declare(strict_types=1);

namespace Drupal\entity_type_behaviors\Plugin\GraphQLCompose\SchemaType;

use Drupal\entity_type_behaviors\Config\BehaviorConfigFactory;
use Drupal\graphql_compose\Plugin\GraphQLCompose\GraphQLComposeEntityTypeInterface;
use Drupal\graphql_compose\Plugin\GraphQLCompose\GraphQLComposeSchemaTypeBase;
use GraphQL\Type\Definition\InterfaceType;
use GraphQL\Type\Definition\ObjectType;
use GraphQL\Type\Definition\Type;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * EntityTypeBehaviorsInterface GraphQL Compose Schema Type.
 *
 * @GraphQLComposeSchemaType(
 *   id = "EntityTypeBehaviorsInterface",
 * )
 */
class EntityTypeBehaviorsInterface extends GraphQLComposeSchemaTypeBase {

  /**
   * The behavior config factory.
   */
  protected BehaviorConfigFactory $behaviorConfigFactory;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): self {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->behaviorConfigFactory = $container->get('entity_type_behaviors.config.factory');
    return $instance;
  }

  /**
   * Returns the GraphQL types provided by this plugin.
   *
   * Defines the EntityTypeBehaviorsInterface interface type.
   *
   * @return array
   *   An array of GraphQL types.
   */
  public function getTypes(): array {
    return [
      new InterfaceType([
        'name' => $this->getPluginId(),
        'description' => (string) $this->t('Entity with behavior settings.'),
        'fields' => fn() => [
          'entity_type_behaviors' => [
            'type' => Type::listOf(static::type('EntityTypeBehaviorInterface')),
            'description' => (string) $this->t('Entity type behaviors for this entity.'),
          ],
        ],
      ]),
    ];
  }

  /**
   * Returns schema extensions for entity types with behaviors.
   *
   * Adds behavior extensions for paragraph and node entity types.
   *
   * @return array
   *   An array of schema extensions.
   */
  public function getExtensions(): array {
    $extensions = parent::getExtensions();
    $extensions = $this->addExtensionsForEntityType('paragraph', $extensions);
    $extensions = $this->addExtensionsForEntityType('node', $extensions);
    return $extensions;
  }

  /**
   * Adds extensions for a specific entity type.
   *
   * @param string $entityTypeId
   *   The entity type ID.
   * @param array $extensions
   *   The existing extensions array.
   *
   * @return array
   *   The updated extensions array.
   */
  private function addExtensionsForEntityType(string $entityTypeId, array $extensions): array {
    $plugin = $this->gqlEntityTypeManager->getPluginInstance($entityTypeId);
    if (!$plugin instanceof GraphQLComposeEntityTypeInterface) {
      return $extensions;
    }

    return array_merge($extensions, $this->getExtensionsByPlugin($plugin, $entityTypeId));
  }

  /**
   * Gets extensions for a specific entity type plugin.
   *
   * Creates extensions for each bundle with enabled behaviors.
   *
   * @param \Drupal\graphql_compose\Plugin\GraphQLCompose\GraphQLComposeEntityTypeInterface $plugin
   *   The entity type plugin.
   * @param string $entityTypeId
   *   The entity type ID.
   *
   * @return array
   *   An array of extensions.
   */
  public function getExtensionsByPlugin(GraphQLComposeEntityTypeInterface $plugin, string $entityTypeId): array {
    $extensions = [];
    $bundles = $plugin->getBundles();

    foreach ($bundles as $bundle) {
      $bundleEntity = $bundle->getEntity();
      $bundleId = $bundleEntity->id();

      $enabledBehaviors = $this->getEnabledBehaviorsForBundle($entityTypeId, $bundleId, $this->behaviorConfigFactory);
      if (empty($enabledBehaviors)) {
        continue;
      }

      $unionTypeName = 'EntityTypeBehaviors' . $this->pascalCase($entityTypeId) . $this->pascalCase($bundleId);
      $extensions[] = $this->createExtensionType($bundle->getTypeSdl(), $unionTypeName, $entityTypeId);
    }

    return $extensions;
  }

  /**
   * Creates an extension type for a bundle.
   *
   * @param string $typeName
   *   The GraphQL type name.
   * @param string $unionTypeName
   *   The union type name for behaviors.
   * @param string $entityTypeId
   *   The entity type ID.
   *
   * @return \GraphQL\Type\Definition\ObjectType
   *   The created extension type.
   */
  private function createExtensionType(string $typeName, string $unionTypeName, string $entityTypeId): ObjectType {
    return new ObjectType([
      'name' => $typeName,
      'interfaces' => fn() => [
        static::type('EntityTypeBehaviorsInterface'),
      ],
      'fields' => fn() => [
        'entity_type_behaviors' => [
          'type' => Type::listOf(static::type($unionTypeName)),
          'description' => (string) $this->t('Behaviors for this @entity_type.', ['@entity_type' => $entityTypeId]),
        ],
      ],
    ]);
  }

  /**
   * Gets enabled behaviors for a specific entity type and bundle.
   *
   * @param string $entityTypeId
   *   The entity type ID.
   * @param string $bundleId
   *   The bundle ID.
   * @param \Drupal\entity_type_behaviors\Config\BehaviorConfigFactory $configFactory
   *   The behavior config factory.
   *
   * @return array
   *   An array of enabled behaviors.
   */
  protected function getEnabledBehaviorsForBundle(string $entityTypeId, string $bundleId, BehaviorConfigFactory $configFactory): array {
    return $configFactory->getEnabledConfiguredBehaviors($entityTypeId, $bundleId);
  }

  /**
   * Converts a string to PascalCase.
   *
   * @param string $string
   *   The string to convert.
   *
   * @return string
   *   The PascalCase string.
   */
  protected function pascalCase(string $string): string {
    return str_replace(' ', '', ucwords(str_replace('_', ' ', $string)));
  }

}
